/* grp.c was originally generated by the autoSql program, which also 
 * generated grp.h and grp.sql.  This module links the database and
 * the RAM representation of objects. */

#include "common.h"
#include "linefile.h"
#include "dystring.h"
#include "jksql.h"
#include "grp.h"

static char const rcsid[] = "$Id: grp.c,v 1.3 2004/04/01 03:22:37 markd Exp $";

void grpStaticLoad(char **row, struct grp *ret)
/* Load a row from grp table into ret.  The contents of ret will
 * be replaced at the next call to this function. */
{
int sizeOne,i;
char *s;

ret->name = row[0];
ret->label = row[1];
ret->priority = atof(row[2]);
}

struct grp *grpLoad(char **row)
/* Load a grp from row fetched with select * from grp
 * from database.  Dispose of this with grpFree(). */
{
struct grp *ret;
int sizeOne,i;
char *s;

AllocVar(ret);
ret->name = cloneString(row[0]);
ret->label = cloneString(row[1]);
ret->priority = atof(row[2]);
return ret;
}

struct grp *grpLoadAll(char *fileName) 
/* Load all grp from a tab-separated file.
 * Dispose of this with grpFreeList(). */
{
struct grp *list = NULL, *el;
struct lineFile *lf = lineFileOpen(fileName, TRUE);
char *row[3];

while (lineFileRow(lf, row))
    {
    el = grpLoad(row);
    slAddHead(&list, el);
    }
lineFileClose(&lf);
slReverse(&list);
return list;
}

struct grp *grpCommaIn(char **pS, struct grp *ret)
/* Create a grp out of a comma separated string. 
 * This will fill in ret if non-null, otherwise will
 * return a new grp */
{
char *s = *pS;
int i;

if (ret == NULL)
    AllocVar(ret);
ret->name = sqlStringComma(&s);
ret->label = sqlStringComma(&s);
ret->priority = sqlFloatComma(&s);
*pS = s;
return ret;
}

struct grp *grpLoadByQuery(struct sqlConnection *conn, char *query)
/* Load all grp from table that satisfy the query given.  
 * Where query is of the form 'select * from example where something=something'
 * or 'select example.* from example, anotherTable where example.something = 
 * anotherTable.something'.
 * Dispose of this with grpFreeList(). */
{
struct grp *list = NULL, *el;
struct sqlResult *sr;
char **row;

sr = sqlGetResult(conn, query);
while ((row = sqlNextRow(sr)) != NULL)
    {
    el = grpLoad(row);
    slAddHead(&list, el);
    }
slReverse(&list);
sqlFreeResult(&sr);
return list;
}

void grpFree(struct grp **pEl)
/* Free a single dynamically allocated grp such as created
 * with grpLoad(). */
{
struct grp *el;

if ((el = *pEl) == NULL) return;
freeMem(el->name);
freeMem(el->label);
freez(pEl);
}

void grpFreeList(struct grp **pList)
/* Free a list of dynamically allocated grp's */
{
struct grp *el, *next;

for (el = *pList; el != NULL; el = next)
    {
    next = el->next;
    grpFree(&el);
    }
*pList = NULL;
}

void grpOutput(struct grp *el, FILE *f, char sep, char lastSep) 
/* Print out grp.  Separate fields with sep. Follow last field with lastSep. */
{
int i;
if (sep == ',') fputc('"',f);
fprintf(f, "%s", el->name);
if (sep == ',') fputc('"',f);
fputc(sep,f);
if (sep == ',') fputc('"',f);
fprintf(f, "%s", el->label);
if (sep == ',') fputc('"',f);
fputc(sep,f);
fprintf(f, "%f", el->priority);
fputc(lastSep,f);
}

/* -------------------------------- End autoSql Generated Code -------------------------------- */

int grpCmpPriority(const void *va, const void *vb)
/* Compare to sort based on priority. */
{
const struct grp *a = *((struct grp **)va);
const struct grp *b = *((struct grp **)vb);
double dif = a->priority - b->priority;
if (dif < 0) return -1;
if (dif > 0) return 1;
return 0;
}

